#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
from typing import Optional

from PySide6.QtWidgets import QApplication, QMessageBox, QDialog

from hatch_pyside.gui.ui_project_chooser_dlg import Ui_ProjectChooser

from hatch_pyside.gui.mainwindow import MainWindow
from hatch_pyside import config


def main():
    app = QApplication()
    try:
        name, projects = config.get_project_folders()
    except OSError:
        QMessageBox(QMessageBox.Icon.Critical,
                    'Fatal',
                    'No pyproject.toml file found',
                    informativeText='The current working directory must '
                                    'contain a pyproject.toml file').exec()
        return 2
    except KeyError:
        QMessageBox(QMessageBox.Icon.Critical,
                    'Fatal',
                    'pyproject.toml has no project name',
                    informativeText="The pyproject.toml file must "
                    "at least contain [project] name=...").exec()
        return 2
    if projects is None or len(projects) == 0:
        QMessageBox(QMessageBox.Icon.Critical,
                    'Fatal',
                    'No hatch-pyside configuration found',
                    informativeText='hatch-pyside plugin must be configured in '
                                    'pyproject.toml or hatch.toml').exec()
        return 1

    folder = choose_project(projects)
    if folder is None:
        return 0

    main_wnd = MainWindow(projects, folder, name)
    main_wnd.show()

    return app.exec()

def choose_project(projects: list[str]) -> Optional[str]:
    if len(projects) == 1:
        return projects[0]
    dlg = DialogChooser(projects)
    if dlg.exec() == 0:
        return None
    return dlg.projectList.selectedItems()[0].text()


class DialogChooser(QDialog, Ui_ProjectChooser):
    def __init__(self, projects):
        super().__init__()
        self.setupUi(self)
        self.projectList.addItems(projects)
        self.projectList.setCurrentRow(0)


if __name__ == '__main__':
    main()


#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
from typing import Any

from hatchling.builders.hooks.plugin.interface import BuildHookInterface
from ..build import build, clean

class PysideBuildHook(BuildHookInterface):
    PLUGIN_NAME = 'pyside'

    def clean(self, _versions: list[str]) -> None:
        clean(self.root)

    def initialize(self, version: str, build_data: dict[str, Any]) -> None:
        build(self.root)
        build_data['artifacts'].append('ui_*.py')


#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
from typing import Any, Optional

try:
    # noinspection PyUnresolvedReferences
    import tomllib
except ImportError:
    # noinspection PyUnresolvedReferences
    import tomli as tomllib


def get_values(cfg: dict[str, Any]) -> Optional[list[str]]:
    try:
        return  cfg['build']['targets']['wheel']['hooks']['pyside']['folders']
    except KeyError:
        return cfg['build']['hooks']['pyside']['folders']


def get_project_folders() -> tuple[str, Optional[list[str]]]:
    """ Extract hatch-pyside config from pyproject.toml or hatch.toml """
    name = None
    try:
        with open('pyproject.toml', 'rb') as fd:
            json = tomllib.load(fd)
            name = json['project']['name']
            return name, get_values(json['tool']['hatch'])
    except KeyError:
        if name is None:
            raise
        try:
            with open('hatch.toml', 'rb') as fd:
                return name, get_values(tomllib.load(fd))
        except (OSError, KeyError):
            return name, None



#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
import shutil
import subprocess
import sysconfig

__all__ = ['command', 'build', 'clean']
command: list[str]


def run(cmd, folder) -> None:
    subprocess.run(command + [cmd, folder],
                   stdin=subprocess.DEVNULL,
                   capture_output=True,
                   text=True,
                   check=True,
                   creationflags=subprocess.CREATE_NO_WINDOW,
                   )


def build(folder: str) -> None:
    run('build', folder)


def clean(folder: str) -> None:
    run('clean', folder)


def _build_command() -> str:
    prog = 'pyside6-project'
    cmd = shutil.which(prog)
    if cmd is None:
        visited = set()
        for d in (sysconfig.get_path('scripts', scheme)
                  for scheme in sysconfig.get_scheme_names()):
            if d in visited:
                continue
            visited.add(d)
            cmd = shutil.which(prog)
            if cmd is not None:
                break
            else:
                raise RuntimeError(f'No installation of {prog} found')
    return cmd


command = [_build_command()]



#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
import json
import os
from pathlib import Path
from typing import Optional

from PySide6.QtCore import Slot
from PySide6.QtGui import QCloseEvent
from PySide6.QtWidgets import QMainWindow, QApplication, QDialog, QMessageBox, QListWidget

import hatch_pyside
import hatch_pyside.gui.app as app
from hatch_pyside import __version__
from hatch_pyside.gui.ui_about import Ui_About
from hatch_pyside.gui.ui_mainwindow import Ui_MainWindow


class MainWindow(QMainWindow, Ui_MainWindow):
    files: list[str]
    project_files: list[str]
    dirty: bool
    json_file: Optional[str] = None

    def __init__(self, projects: list[str], folder: str, name: str):
        super().__init__()
        self.setupUi(self)
        self.projects = projects
        self.project = folder
        self.setWindowTitle(f"hatch-pyside - {folder}")
        if len(projects) <= 1:
            self.action_Change_project_folder.setVisible(False)
        self.reload()
        self.proj_name = name

    @Slot()
    def about_qt(self):
        # noinspection PyUnresolvedReferences
        QApplication.instance().aboutQt()

    @Slot()
    def about(self):
        About().exec()

    @Slot()
    def change_project(self):
        self.project = app.choose_project(self.projects)
        self.setWindowTitle(f"hatch-pyside - {self.project}")
        self.reload()

    @Slot()
    def reload(self):
        self.files = [f for f in os.listdir(self.project)
                      if os.path.isfile(os.path.join(self.project, f))]
        for f in self.files:
            if os.path.normcase(f).endswith('.pyproject'):
                with open(Path(self.project) / f) as fd:
                    self.project_files = json.load(fd)['files']
                    self.json_file = f
                    break
        else:
            self.json_file = None
            self.project_files = []
        self.project_files_widget.clear()
        self.files_widget.clear()
        self.project_files_widget.addItems(self.project_files)
        items = [f for f in self.files if f not in self.project_files]
        self.files_widget.addItems(items)
        self.dirty = False

    @Slot()
    def build(self):
        hatch_pyside.build(self.project)
        self.reload()

    @Slot()
    def clean(self):
        hatch_pyside.clean(self.project)
        self.reload()

    @Slot()
    def remove(self):
        self.dirty |= move_sel(self.project_files_widget, self.files_widget)

    @Slot()
    def add(self):
        self.dirty |= move_sel(self.files_widget, self.project_files_widget)

    @Slot()
    def save(self):
        print("Saving")
        self.project_files = sorted(
            [self.project_files_widget.item(row).text()
             for row in range(self.project_files_widget.count())])
        json_file = self.json_file
        if json_file is None:
            json_file = f'{self.proj_name}.pyproject'
        with open(Path(self.project) / json_file, 'w') as fd:
            # noinspection PyTypeChecker
            json.dump({'files': self.project_files}, fd)
            self.json_file = json_file
            self.dirty = False

    def closeEvent(self, event: QCloseEvent):
        if self.dirty:
            buttons = (QMessageBox.StandardButton.Save
                       | QMessageBox.StandardButton.Discard
                       | QMessageBox.StandardButton.Cancel)
            cr = QMessageBox(QMessageBox.Icon.Warning, "Save?",
                           "The project file has unsaved changes"
                           "do you want to quit anyway?",
                           standardButtons=buttons).exec()
            if cr == QMessageBox.StandardButton.Cancel:
                event.ignore()
                return
            elif cr == QMessageBox.StandardButton.Save:
                self.save()
        event.accept()


def move_sel(src: QListWidget, dest: QListWidget) -> bool:
    sel = src.selectedItems()
    if len(sel) == 0:
        return False
    for item in dest.selectedItems():
        item.setSelected(False)
    for item in sel:
        it = src.takeItem(src.row(item))
        dest.addItem(it)
        it.setSelected(True)
    return True


def clear_selection(list_widget: QListWidget) -> None:
    for item in list_widget.selectedItems():
        item.setSelected(False)


class About(QDialog, Ui_About):
    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.version.setText(__version__)
